import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * Table of opened Images.
 * @author Koutaro Abe ( kabe@cris.hokudai.ac.jp )
 * @version 26-October-2010
 */
@SuppressWarnings("serial")
public class ImageList {
			
	public JTable jt;
	ImagelistTableModel table;
	ArrayList<Integer> windowId;
	final int COLUMN_CHECK = 0;
	final int COLUMN_TITLE = 1;
	final int COLUMN_TYPE = 2;
	String[] columnname = { "Check", "Image title", "Type"};
	Object[][] data = { { false, "title", "description" } };
	
	ImageList() {
		windowId = new ArrayList<Integer>();
		table = new ImagelistTableModel( columnname );
		jt = new JTable( table );
		if (IJ.isLinux()) jt.setBackground(Color.white);
		jt.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		jt.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );
		resizeColumn( COLUMN_CHECK );
		refresh();
	}

	public void resizeColumn( int column ){
		if( jt.getRowCount()==0 ) {
			table.addRow( new Object[]{ (Boolean) true, "IMAGE_TITLE_012345678901", "XXbit XXX x XXX xXXX....." } );
			for( int i=0; i<columnname.length; i++ )
				resizeColumn( jt, i );
			table.removeRow( 0 );
		} else
			resizeColumn( jt, column );
	}
	void resizeColumn( JTable table, int column ) {
		final JTable ftable = table;
		TableColumnModel columnModel = ftable.getColumnModel();
//		IJ.log( "tablecolumnmodel = "+columnModel );
		int maxwidth = 0;
		for( int row=0; row<ftable.getRowCount(); row++ ){
			TableCellRenderer rend = ftable.getCellRenderer( row, column );
			Component comp = rend.getTableCellRendererComponent(
					ftable,
					ftable.getValueAt( row, column ),
					false, false,
					row, column );
			maxwidth = Math.max( comp.getPreferredSize().width, maxwidth );
//			IJ.log( "resize:"+maxwidth );
		}
		TableColumn tablecolumn = columnModel.getColumn( column );
		tablecolumn.setMaxWidth( maxwidth*2 );		
		tablecolumn.setPreferredWidth( maxwidth+1 );
		ftable.revalidate();
	};
	public Object getValueAt( int row, int col ) {
		return table.getValueAt( row, col );
	};
	public String getTitleAt( int row ) {
		return (String) table.getValueAt( row, COLUMN_TITLE );
	};
	public void add( ImagePlus imp ) {
		add( imp, false );
	};
	public void add( ImagePlus imp, boolean check ) {
		if( imp==null )
			return;
		String title = imp.getTitle();
		String type = imp.getBitDepth()+"bit: "+imp.getWidth()+" x "+imp.getHeight();
		if( imp.getStackSize()>1 )
			type += " x "+imp.getStackSize();
		if( title==null || title.length()==0 ){
			title = WindowManager.getUniqueName( "Image" );
			imp.setTitle( title );
		}
		if( table.getRowCount()>0 )
			for( int r=0; r<table.getRowCount(); r++ )
				if ( table.getValueAt( r, COLUMN_TITLE ).toString().equals(title) ) {
					title = WindowManager.getUniqueName( title );
					imp.setTitle( title );
				}
		windowId.add( imp.getID() );
		table.addRow( new Object[]{ (Boolean) check, title, type } );
	};
	public void remove( int id ) {
		int i = windowId.indexOf(id);
		if( i==-1 )
			return;
		windowId.remove( i );
		table.removeRow( i );
	}
	public void refresh() {
		if( table!=null ) {
			while( table.getRowCount()>0 )
				table.removeRow( 0 );
			windowId.clear();
		}
		int[] windowId = WindowManager.getIDList();
		if( windowId!=null && windowId.length>0 )
			for( int r=0; r<windowId.length; r++ ) {
				ImagePlus imp = WindowManager.getImage( windowId[r] );
				if( imp!=null )
					add( imp );
			}
	}
	public ImagePlus idToImagePlus( int id ) {
		if( id>=0 && id<windowId.size() )
			return WindowManager.getImage( windowId.get( id ) ); 
		else
			return null;
	}
	ListSelectionModel getSelectionModel() {
		return jt.getSelectionModel();
	}
	int[] getSelectedRows() {
		return jt.getSelectedRows();
	}
	int getSelectedRow() {
		return jt.getSelectedRow();
	}
	ArrayList<Integer> getCheckedRows() {
		ArrayList<Integer> checked = new ArrayList<Integer>();
		for( int i=0; i<table.getRowCount(); i++ )
			if( (Boolean) table.getValueAt( i, COLUMN_CHECK )==true )
				checked.add( i );
		return checked;
	}
	ArrayList<ImagePlus> getCheckedImp() {
		ArrayList<ImagePlus> imp = new ArrayList<ImagePlus>();
		Iterator<Integer> i = getCheckedRows().iterator();
		while( i.hasNext() )
			imp.add( idToImagePlus( i.next() ) );
		return imp;
	}
	ArrayList<ImagePlus> getAllImp() {
		ArrayList<ImagePlus> imp = new ArrayList<ImagePlus>();
		for( int i=0; i<table.getRowCount(); i++ )
			imp.add( idToImagePlus( i ) );
		return imp;
	}
	ArrayList<ImagePlus> getStacksImp() {
		ArrayList<ImagePlus> imp = new ArrayList<ImagePlus>();
		for( int i=0; i<table.getRowCount(); i++ )
			if( idToImagePlus( i ).getStackSize()==1 )
				imp.add( idToImagePlus( i ) );
		return imp;
	}
	ArrayList<ImagePlus> getSlicesImp() {
		ArrayList<ImagePlus> imp = new ArrayList<ImagePlus>();
		for( int i=0; i<table.getRowCount(); i++ )
			if( idToImagePlus( i ).getStackSize()==1 )
				imp.add( idToImagePlus( i ) );
		return imp;
	}
	
	String[] getLabels(){
		String[] label = new String[table.getRowCount()];
		for( int i=0; i<table.getRowCount(); i++ )
			label[i] = (String) table.getValueAt( i, COLUMN_TITLE );
		return label;
	}

	void checkAllRows( boolean flag ) {
		for( int i=0; i<table.getRowCount(); i++ )
			table.setValueAt( flag, i, COLUMN_CHECK );
		return;
	}
	int checkState() {
		int num = table.getRowCount();
		int checked=0;
		for( int i=0; i<num; i++ )
			if( (Boolean) table.getValueAt( i, COLUMN_CHECK )==true )
				checked++;
		if( checked==0 )
			return 0;
		if( checked==num )
			return 1;
		return -1;
	}

	class ImagelistTableModel extends DefaultTableModel {
		ImagelistTableModel( String[] columns ) {
			super( new Object[0][], columns );
		}
		public Class getColumnClass( int col ) {
			try {
				return getValueAt( 0, col ).getClass();
			} catch( Exception e ) {
				return String.class;
			}
		}
	}
}
